/*
 * Copyright (c) 1998 Thomas Nystrm and Stacken Computer Club
 * (Royal Institute of Technology, Stockholm, Sweden).
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Kungliga Tekniska
 *      Hgskolan and its contributors.
 * 
 * 4. Neither the name of the Institute nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "common.h"
#pragma hdrstop

#include "ktelnet.rh"
#include "newconn.h"
#include "telnet.h"
#include "serconn.h"
#include "ftpconn.h"
#include "emul.rh"

#define GHD_MAXSTR 64

//****************************************************************

class PrpGetHostPage;
class PrpGetSerialPage;
class PrpGetFTPPage;

class PrpGetHostDialog : public TPropertySheet
{
        friend PrpGetHostPage;
        friend PrpGetSerialPage;
	friend PrpGetFTPPage;

      public:
	PrpGetHostDialog(TWindow *parent, Registry *_reg, const char *user = 0);
	~PrpGetHostDialog();
	
	virtual void SetupWindow();
	const char *GetHostName();
	const char *GetUserName();
	const char *GetPortName();
	const char *GetSerialPort();
	void SetConnectionType(ConnectionType _ct);
	ConnectionType GetConnectionType();
	void AddStrings(TComboBoxData &cbd, int ResId);
	int Execute();

      private:
	PHOSTENT host;
	char *hn;
	bool verified;
	struct {
		TComboBoxData d;
		char user[GHD_MAXSTR];
		TComboBoxData p;
	} HostBuffer;
	struct {
		TComboBoxData Port;
		char settings[GHD_MAXSTR];
	} SerialBuffer;
	struct {
		TComboBoxData d;
		char user[GHD_MAXSTR];
	} FTPHostBuffer;
	ConnectionType ct;
	bool SaveOnClose;
};

class PrpGetHostPage : TPropertyPage
{
      public:
	PrpGetHostPage(PrpGetHostDialog *ps, Registry *reg);

      protected:
        virtual int Apply(TNotify far&);
	PrpGetHostDialog *GHD;
	virtual int SetActive(TNotify &n);
};

class PrpGetSerialPage : TPropertyPage
{
      public:
	PrpGetSerialPage(PrpGetHostDialog *ps, Registry *reg);
	~PrpGetSerialPage();
	virtual void SetupWindow();

      protected:
	void EvPortChanged();
	void CmSerialSetup();

	virtual int SetActive(TNotify &n);
        virtual int Apply(TNotify far&);

	PrpGetHostDialog *GHD;
	Registry *rps;

DECLARE_RESPONSE_TABLE(PrpGetSerialPage);
};

class PrpGetFTPPage : TPropertyPage
{
      public:
	PrpGetFTPPage(PrpGetHostDialog *ps, Registry *reg);

      protected:
        virtual int Apply(TNotify far&);
	PrpGetHostDialog *GHD;
	virtual int SetActive(TNotify &n);
};

//****************************************************************

PrpGetHostDialog::PrpGetHostDialog(TWindow *parent, Registry *reg,
	const char *user)
	: TPropertySheet(parent, ResString(parent, IDS_CONNTO), 0, false, 
		PSH_DEFAULT | PSH_NOAPPLYNOW)
{
	hn = 0;
	verified = false;
	HostBuffer.user[0] = 0;
	FTPHostBuffer.user[0] = 0;
	if (user) if (user[0]) {
		strcpy_truncate(HostBuffer.user, user, GHD_MAXSTR);
		strcpy_truncate(FTPHostBuffer.user, user, GHD_MAXSTR);
		}

	AddStrings(HostBuffer.p, IDS_PORTS);
	AddStrings(SerialBuffer.Port, IDS_COMPORTS);

	// Must match order in PrpGetHostDialog::SetConnectionType()
	new PrpGetHostPage(this, reg);
	new PrpGetSerialPage(this, reg);
	new PrpGetFTPPage(this, reg);
}

PrpGetHostDialog::~PrpGetHostDialog()
{
	if (hn)
		delete[] hn;
}

void
PrpGetHostDialog::SetupWindow()
{
	TPropertySheet::SetupWindow();
}

void
PrpGetHostDialog::SetConnectionType(ConnectionType _ct)
{
	// Must match order in PrpGetHostDialog::PrpGetHostDialog()

	switch (_ct) {
		case ctTelnet:
			HeaderInfo.nStartPage = 0;
			ct = _ct;
			break;
		case ctSerial:
			HeaderInfo.nStartPage = 1;
			ct = _ct;
			break;
		case ctFTP:
			HeaderInfo.nStartPage = 2;
			ct = _ct;
			break;
		}
}

int
PrpGetHostDialog::Execute()
{
	// Det r inte dokumenterat vad TPropertySheet::Execute()
	// returnerar, anvnd en flagga fr att veta om vi skall
	// spara datat!

	ct = ctNone;
	SaveOnClose = false;

	TPropertySheet::Execute();
	
	return ((ct != ctNone)&&SaveOnClose)?IDOK:IDCANCEL;
}

void
PrpGetHostDialog::AddStrings(TComboBoxData &cbd, int ResId)
{
	char *res, *cp, *cpn;
	bool first = true;
	
	res = GetNewStringResource(GetApplication(), ResId);

	for (cp = cpn = res; *cp; cp++) {
		if (*cp == '|') {
			*cp = 0;
			cbd.AddString(cpn, first);
			cpn = cp+1;
			first = false;
			}
		}
	if (cpn != cp)
		cbd.AddString(cpn, first);

	delete[] res;
}

const char *
PrpGetHostDialog::GetHostName()
{
	if (verified && hn)
		return hn;

	switch (ct) {
		case ctTelnet:
			return HostBuffer.d.GetSelection().c_str();
		case ctFTP:
			return FTPHostBuffer.d.GetSelection().c_str();
		}
	return "";
}	

const char *
PrpGetHostDialog::GetUserName()
{
	switch (ct) {
		case ctTelnet:
			return HostBuffer.user;
		case ctFTP:
			return FTPHostBuffer.user;
		}
	return "";
}	

const char *
PrpGetHostDialog::GetPortName()
{
	switch (ct) {
		case ctTelnet:
			return HostBuffer.p.GetSelection().c_str();
		}
	return "";
}

const char *
PrpGetHostDialog::GetSerialPort()
{
	return SerialBuffer.Port.GetSelection().c_str();
}

ConnectionType
PrpGetHostDialog::GetConnectionType()
{
	return ct;
}

//****************************************************************


PrpGetHostPage::PrpGetHostPage(PrpGetHostDialog *ps, Registry *reg)
		: TPropertyPage(ps, IDD_GETHOST)
{
	GHD = ps;
	new InputHistory(this, IDC_HOST, new Registry(reg, "Host"));
	new TEdit(this, IDC_USER, GHD_MAXSTR);
	new WXComboBox(this, IDC_PORT);

	SetTransferBuffer(&GHD->HostBuffer);
}

int
PrpGetHostPage::Apply(TNotify far &)
{
	const char *cp;

	Transfer(&GHD->HostBuffer, tdGetData);

	if (GHD->ct != ctTelnet)
		return PSNRET_NOERROR;

	cp = GHD->HostBuffer.d.GetSelection().c_str();
	if (*cp != '\\') {
		HCURSOR OldCur =
			::SetCursor(::LoadCursor(0, IDC_WAIT));
		GHD->host = gethostbyname(cp);
		::SetCursor(OldCur);

		if (GHD->host == NULL) {
			MessageBox(ResString((TModule *)0, IDS_UNKHST),
				errorHeader, MB_OK|MB_ICONEXCLAMATION);
			return PSNRET_INVALID;
			}
		GHD->hn = strnewdup(GHD->host->h_name);
		GHD->verified = true;
		}

	GHD->SaveOnClose = true;
	return PSNRET_NOERROR;
}

int
PrpGetHostPage::SetActive(TNotify &n)
{
	TPropertyPage::SetActive(n);

	GHD->ct = ctTelnet;
	return 0;
}

//****************************************************************

DEFINE_RESPONSE_TABLE1(PrpGetSerialPage, TPropertyPage)
	EV_CBN_SELCHANGE(IDC_COMPORT, EvPortChanged),
	EV_COMMAND(IDC_CTSETUP, CmSerialSetup),
END_RESPONSE_TABLE;

PrpGetSerialPage::PrpGetSerialPage(PrpGetHostDialog *ps, Registry *reg)
		: TPropertyPage(ps, IDD_GETSERIAL)
{
	GHD = ps;
	rps = new Registry(reg, "SerialConnection");
	new WXComboBox(this, IDC_COMPORT);
	(new TStatic(this, IDC_CTSETTING, GHD_MAXSTR))->EnableTransfer();

	SetTransferBuffer(&GHD->SerialBuffer);
}

PrpGetSerialPage::~PrpGetSerialPage()
{
	delete rps;
}

void
PrpGetSerialPage::SetupWindow()
{

	TPropertyPage::SetupWindow();

	try {
		const char *s;
		WXComboBox *cb;

		rps->Open();
		s = rps->GetString("LastUsed");
		rps->Close();
		cb = dynamic_cast<WXComboBox *>(ChildWithId(IDC_COMPORT));
		if (cb)
			cb->SetSelString(s, 0);
		}
	catch (RegistryFail) {
		rps->Close();
		}
	
	EvPortChanged();
}

void
PrpGetSerialPage::EvPortChanged()
{
	WXComboBox *cb;
	char buf[128], *dev;
	SERPAR SerialPars;

	cb = dynamic_cast<WXComboBox *>(ChildWithId(IDC_COMPORT));
	if (cb == NULL) {
		MessageBeep(-1);
		return;
		}

	if ((dev = cb->GetSelection()) == NULL) {
		MessageBeep(-1);
		return;
		}

	if (!GetSerialPars(rps, dev, SerialPars))
		SetDlgItemText(IDC_CTSETTING, "");
	else {
		SerialSetup(this, &SerialPars).
			GetInfoString(buf, sizeof(buf));
		SetDlgItemText(IDC_CTSETTING, buf);
		}

	delete[] dev;
}

int
PrpGetSerialPage::SetActive(TNotify &n)
{
	TPropertyPage::SetActive(n);

	GHD->ct = ctSerial;
	return 0;
}

int
PrpGetSerialPage::Apply(TNotify far &)
{
	const char *cp;

	Transfer(&GHD->SerialBuffer, tdGetData);

	if (GHD->ct != ctSerial)
		return PSNRET_NOERROR;

	cp = GHD->SerialBuffer.Port.GetSelection().c_str();
	
	try {
		rps->Open(true);
		rps->SetString("LastUsed", cp);
		rps->Close();
		}
	catch (RegistryFail) {
		rps->Close();
		}

	GHD->SaveOnClose = true;

	return PSNRET_NOERROR;
}

void
PrpGetSerialPage::CmSerialSetup()
{
	WXComboBox *cb;
	SERPAR SerialPars;
	char *dev;
	char buf[128];

	cb = dynamic_cast<WXComboBox *>(ChildWithId(IDC_COMPORT));

	if (cb == NULL) {
		MessageBeep(-1);
		return;
		}

	if ((dev = cb->GetSelection()) == NULL) {
		MessageBeep(-1);
		return;
		}

	GetSerialPars(rps, dev, SerialPars);

	if (SerialSetup(this, &SerialPars).Execute() == IDOK) {
		try {
			rps->Open(true);
			rps->SetBinData(dev,
				(BYTE *)&SerialPars, sizeof(SerialPars));
			rps->Close();
			}
		catch (RegistryFail) {
			rps->Close();
			MessageBox(ResString(this, IDS_CNSAVREG), errorHeader,
				MB_OK|MB_ICONEXCLAMATION);
			}
		SerialSetup(this, &SerialPars).GetInfoString(buf, sizeof(buf));
		SetDlgItemText(IDC_CTSETTING, buf);
		}

	delete[] dev;
}

//****************************************************************


PrpGetFTPPage::PrpGetFTPPage(PrpGetHostDialog *ps, Registry *reg)
		: TPropertyPage(ps, IDD_GETFTP)
{
	GHD = ps;
	new InputHistory(this, IDC_HOST, new Registry(reg, "FTPHost"));
	new TEdit(this, IDC_USER, GHD_MAXSTR);

	SetTransferBuffer(&GHD->FTPHostBuffer);
}

int
PrpGetFTPPage::Apply(TNotify far &)
{
	const char *cp;

	Transfer(&GHD->FTPHostBuffer, tdGetData);

	if (GHD->ct != ctFTP)
		return PSNRET_NOERROR;

	cp = GHD->FTPHostBuffer.d.GetSelection().c_str();
	if (*cp != '\\') {
		HCURSOR OldCur =
			::SetCursor(::LoadCursor(0, IDC_WAIT));
		GHD->host = gethostbyname(cp);
		::SetCursor(OldCur);

		if (GHD->host == NULL) {
			MessageBox(ResString((TModule *)0, IDS_UNKHST),
				errorHeader, MB_OK|MB_ICONEXCLAMATION);
			return PSNRET_INVALID;
			}
		GHD->hn = strnewdup(GHD->host->h_name);
		GHD->verified = true;
		}

	GHD->SaveOnClose = true;
	return PSNRET_NOERROR;
}

int
PrpGetFTPPage::SetActive(TNotify &n)
{
	TPropertyPage::SetActive(n);

	GHD->ct = ctFTP;
	return 0;
}

//****************************************************************

OpenConnection::OpenConnection(TWindow *parent, Registry *userData)
{
	Parent = parent;
	UserData = userData;
}

OpenConnection::~OpenConnection()
{
}

Connection *
OpenConnection::Execute(const char *host, const char *user, const char *port)
{
	char buf[256];
	Connection *NewConnection = 0;
	PHOSTENT h;
	char *chost = 0;
	ConnectionType ct = ctTelnet;

	if (port) {
		if (strcmp(port, "-FTP-") == 0) {
			ct = ctFTP;
			port = "";
			}
		else if (strcmp(port, "-SERIAL-") == 0) {
			ct = ctSerial;
			port = "";
			}
		}

	PrpGetHostDialog ghd(Parent, UserData);

	if (host == NULL) {
		ghd.SetConnectionType(ct);

		if (ghd.Execute() != IDOK)
			return 0;
		ct = ghd.GetConnectionType();
		if ((ct == ctTelnet) || (ct == ctFTP)) {
			chost = strnewdup(ghd.GetHostName());
			user = ghd.GetUserName();
			port = ghd.GetPortName();
			}
		else if (ct == ctSerial)
			chost = strnewdup(ghd.GetSerialPort());
		}
	else if ((*host == '\\') || (ct == ctSerial))
		chost = strnewdup(host);
	else {
		HCURSOR OldCur = ::SetCursor(::LoadCursor(0, IDC_WAIT));
		h = gethostbyname(host);
		::SetCursor(OldCur);

		if (h == NULL) {
			Parent->MessageBox(ResString((TModule *)0,
				IDS_UNKHST), errorHeader,
				MB_OK|MB_ICONEXCLAMATION);
			return 0;
			}
		chost = strnewdup(h->h_name);
		}

	switch (ct) {
		case ctFTP:
			NewConnection = new FTPConnection(Parent);
			if (NewConnection->Open(chost, user, port)) {
				strcpy_truncate(buf,
					ResString(Parent, IDS_FTP),
					sizeof(buf));
				strcat_truncate(buf, " - ", sizeof(buf));
				strcat_truncate(buf,
					NewConnection->GetConnectionName(),
					sizeof(buf));
				Parent->Parent->SetCaption(buf);
				delete[] chost;
				return NewConnection;
				}
			break;
		case ctTelnet:
			NewConnection = new KTelnetConnection(Parent);

			if (NewConnection->Open(chost, user, port)) {
				strcpy_truncate(buf, header, sizeof(buf));
				strcat_truncate(buf, " - ", sizeof(buf));
				strcat_truncate(buf,
					NewConnection->GetConnectionName(),
					sizeof(buf));
				Parent->Parent->SetCaption(buf);
				delete[] chost;
				return NewConnection;
				}
			break;
		case ctSerial:
			NewConnection = new SerialConnection(Parent,
				UserData);

			if (NewConnection->Open(chost)) {
				strcpy_truncate(buf, header, sizeof(buf));
				strcat_truncate(buf, " - ", sizeof(buf));
				strcat_truncate(buf,
					NewConnection->GetConnectionName(),
					sizeof(buf));
				Parent->Parent->SetCaption(buf);
				delete[] chost;
				return NewConnection;
				}
			break;
		}
	if (NewConnection)
		delete NewConnection;
	if (chost)
		delete[] chost;
	return 0;
}
